/**
 * @author Guillaume Santini
 * @version 1.0
 * Classe entitée permettant de décrire un devis.
 */
import java.util.HashSet;

public class Devis {
	private NumeroteurDeDevis numeroteur ;
	private HashSet<Produit> panier ;
	private DiscountType discount ;
	private TaxationType taxation;
	
	/**
	 * Constructeur par champs initialisé à partir d'une collection de produits.
	 * @param numeroteur le numéroteur de devis
	 */
	public Devis(NumeroteurDeDevis numeroteur, DiscountType discountType, TaxationType taxationType, HashSet<Produit> panier) {
		this.numeroteur = numeroteur;
		this.discount = discountType;
		this.taxation = taxationType;
		this.panier = panier;
	}
	
	/**
	 * Constructeur initialisé avec un panier vide.
	 * @param numeroteur le numéroteur de devis
	 */
	public Devis(NumeroteurDeDevis numeroteur, DiscountType discountType, TaxationType taxationType) {
		this.numeroteur = numeroteur;
		this.discount = discountType;
		this.taxation = taxationType;
		this.panier = new HashSet<Produit>();
	}
	
	/**
	 * Ajoute au contenu du panier un produit.
	 * @param produit le produit ajouté au panier
	 */
	public void ajouterAuPanier(Produit produit) {
		panier.add(produit);
	}
	
	/**
	 * Retire un produit du contenu du panier.
	 * @param produit le produit à retirer du panier
	 */
	public void retirerDuPanier(Produit produit) {
		panier.remove(produit);
	}
	
	public String editionDevis() {
		Double totalHT = 0. ;
		Double totalD  = 0. ;
		Double totalTVA = 0. ;
		String facture = "--------------------------------------------------\n";
		facture += "Devis n° "+numeroteur.numeroDeDevis()+"\n";
		facture += "--------------------------------------------------\n";
		for (Produit p : panier) {
			facture += p+"\n";
			totalHT += p.getPrix() ;
			switch (this.discount) {
				case PREMIUM :
					totalD  += 0.2* p.getPrix();
				case NODISCOUNT :
					totalD  += 0.;
			}	
		}
		Double totalHTD = totalHT-totalD ;
		switch (this.taxation) {
			case TVA20 :
				totalTVA = 0.2 * totalHTD;
			case TVA5 :
				totalTVA = 0.05 * totalHTD;
		}
		Double grandTotal = totalHTD + totalTVA ;
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Total HT",totalHT);
		facture += "--------------------------------------------------\n";
		switch (this.discount) {
			case PREMIUM : 		facture += String.format("%-30s\t\t%10.2f\n","Discount (PREMIUM)",totalD);		break;
			case NODISCOUNT : 	facture += String.format("%-30s\t\t%10.2f\n","Discount (NODISCOUNT)",totalD);	break;
		}
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Total après discount",totalHTD);
		facture += "--------------------------------------------------\n";
		switch (this.taxation) {
			case TVA20 :		facture += String.format("%-30s\t\t%10.2f\n","Taxes (20%)",totalTVA);			break;
			case TVA5 :			facture += String.format("%-30s\t\t%10.2f\n","Taxes (5%)",totalTVA);			break;
		}
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Total TTC",grandTotal);
		return facture;
	}
}
